.include	"tn15def.inc"		; Aktueller Controller ATtiny15L


.equ	RxD	=0	; SW-UART
.equ	TxD	=1	; SW-UART
.equ	FREE	=2
.equ	KEY1	=3
.equ	KEY2	=4
.equ	PORT	=PORTB
.equ	PIN	=PINB

.def	bitcnt	=R16			;bit counter
.def	temp	=R17			;temporary storage register
.def	Txbyte	=R18			;Data to be transmitted
.def	wait	=R19
.def	wait2	=R20
.def	oldkey	=R21


.ORG	0				; *** reset vector 
	rjmp reset

reset:
	ldi	temp,	(1<<TxD)
	out	DDRB,	temp
	ldi	temp,	(1<<RxD)|(1<<KEY1)|(1<<KEY2)|(1<<FREE)
	out	PORTB,	temp
	ldi	oldkey,	(1<<KEY1)|(1<<KEY2)

main:

;DEBUG
;sbi	PORT,	TxD
;nop
;nop
;nop
;nop
;nop
;nop
;nop
;nop

;cbi	PORT,	TxD
;nop
;nop
;nop
;nop
;nop
;nop

;rjmp main
;/DEBUG

	in	temp,	PIN
	andi	temp,	(1<<KEY1)|(1<<KEY2)
	cp	temp,	oldkey
	brne	H_KEY
 retk:	mov	oldkey,	temp
	rjmp	main

h_key:
	ldi	txbyte, '0'
	sbrs	temp,	KEY1
	  ldi	txbyte,	'1'
	sbrs	temp,	KEY2
	  ldi	txbyte,	'2'
	rcall	putchar

	rcall	wait_us

	rjmp	retk


;***************************************************************************
;*
;* "putchar"
;*
;* This subroutine transmits the byte stored in the "Txbyte" register
;* The number of stop bits used is set with the sb constant
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on bit rate
;* Low registers used	:None
;* High registers used	:2 (bitcnt,Txbyte)
;* Pointers used	:None
;*
;***************************************************************************
.equ		sb	=1		;Number of stop bits (1, 2, ...)

putchar:	ldi	bitcnt,9+sb	;1+8+sb (sb is # of stop bits)
		com	Txbyte		;Inverte everything
		sec			;Start bit

putchar0:	brcc	putchar1	;If carry set
		sbi	PORT,TxD	;    send a '0'
		rjmp	putchar2	;else	

putchar1:	cbi	PORT,TxD	;    send a '1'
		nop

putchar2:	rcall UART_delay	;One bit delay
		rcall UART_delay

		lsr	Txbyte		;Get next bit
		dec	bitcnt		;If not all bit sent
		brne	putchar0	;   send next
					;else
		ret			;   return


;***************************************************************************
;*
;* "UART_delay"
;*
;* This delay subroutine generates the required delay between the bits when
;* transmitting and receiving bytes. The total execution time is set by the
;* constant "b":
;*
;*	3b + 7 cycles (including rcall and ret)
;*
;* Number of words	:4 including return
;* Low registers used	:None
;* High registers used	:1 (temp)
;* Pointers used	:None
;*
;***************************************************************************
; Some b values: 	(See also table in Appnote documentation)
;
; 1 MHz crystal:
;   9600 bps - b=14
;  19200 bps - b=5
;  28800 bps - b=2
;
; 2 MHz crystal:
;  19200 bps - b=14
;  28800 bps - b=8
;  57600 bps - b=2

; 4 MHz crystal:
;  19200 bps - b=31
;  28800 bps - b=19
;  57600 bps - b=8
; 115200 bps - b=2

.equ	b	=16	;ca. 9600 bps @ 1 MHz int clock


UART_delay:	ldi	wait,b
UART_delay1:	dec	wait
		brne	UART_delay1

		ret

;***************************************************************************
;*
;* "getchar"
;*
;* This subroutine receives one byte and returns it in the "Rxbyte" register
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on when data arrives
;* Low registers used	:None
;* High registers used	:2 (bitcnt,Rxbyte)
;* Pointers used	:None
;*
;***************************************************************************

getchar:	ldi 	bitcnt,9	;8 data bit + 1 stop bit

getchar1:	sbis 	PIN,	RxD	;Wait for start bit
		rjmp 	getchar1

		rcall UART_delay	;0.5 bit delay

getchar2:	rcall UART_delay	;1 bit delay
		rcall UART_delay		

		clc			;clear carry
		sbis 	PIN,	RxD	;if RX pin high
		sec			;

		dec 	bitcnt		;If bit is stop bit
		breq 	getchar3	;   return
					;else
		ror 	Txbyte		;   shift bit into Rxbyte
		rjmp 	getchar2	;   go get next

getchar3:	ret


;***************************************************************************	
wait_us:	; Wartet ca. 100*WAIT s @ 4 MHz
		; ndert: WAIT, WAIT2
		
	ldi	WAIT2,	0x83
  __long_delay0:
	subi	WAIT2,	1	;2 cycles
	brne	__long_delay0 	;2 cycles
	subi	WAIT,	1
	brne	wait_us 	;2 cycles
	ret

